#include <stdio.h>
#include <stdlib.h>

struct container {
  int count, size;
  int *a;
};

typedef struct container * array_container;

array_container make_array_container() {
  array_container ac;

  ac = (array_container)malloc(sizeof(struct container));
  ac->count = 0;
  ac->size = 1;
  ac->a = (int*)malloc(ac->size * sizeof(int));

  return ac;
}

void copy_array(int* to, int* from, int len) {
  int i;
  for (i = len; i-- > 0; ) {
    to[i] = from[i];
  }
}

void maybe_grow_array(array_container ac) {
  int *new_a;
  if (ac->count + 1 >= ac->size) {
    ac->size *= 2;
    new_a = (int*)malloc(ac->size * sizeof(int));
    copy_array(new_a, ac->a, ac->count);
    ac->a = new_a;
  }
}

void add_to_front(array_container ac, int i) {
  maybe_grow_array(ac);

  copy_array(ac->a+1, ac->a, ac->count);
  ac->a[0] = i;
  ac->count++;
}

void add_to_back(array_container ac, int i) {
  maybe_grow_array(ac);

  ac->a[ac->count] = i;
  ac->count++;
}

int remove_from_front(array_container ac) {
  int v;

  v = ac->a[0];
  --ac->count;
  copy_array(ac->a, ac->a+1, ac->count);

  return v;
}

int remove_from_back(array_container ac) {
  --ac->count;
  return ac->a[ac->count];
}


int get_count(array_container ac) {
  return ac->count;
}

int get_value(array_container ac, int i) {
  return ac->a[i];
}

/* ---------------------------------------- */

int bsearch_array(array_container ac, int i)
/* Looks for `i' in `ac', returns either an index
   or -1 as "not found" */
{
  int n = 0;
  int m = get_count(ac) - 1;

  while (n != m) {
    int p = (n + m) / 2;
    int v = get_value(ac, p);
    
    if (v > i)
      m = p;
    else if (v < i)
      n = p + 1;
    else
      return p;
  }

  if (get_value(ac, n) == i)
    return n;

  return -1;
}

/* ---------------------------------------- */

/* Tests */

void failed(int a, int b) {
  printf("test failed; expected %d, got %d\n", b, a);
  exit(1);
}

void check_int(int a, int b) {
  if (a != b) failed(a, b);
}

void test() {
  array_container ac;

  ac = make_array_container();
  add_to_back(ac, 5);
  add_to_back(ac, 17);
  add_to_back(ac, 50);
  add_to_back(ac, 88);
  add_to_back(ac, 89);
  add_to_back(ac, 91);
  add_to_back(ac, 92);
  
  check_int(bsearch_array(ac, 0), -1);
  check_int(bsearch_array(ac, 5), 0);
  check_int(bsearch_array(ac, 15), -1);
  check_int(bsearch_array(ac, 17), 1);
  check_int(bsearch_array(ac, 50), 2);
  check_int(bsearch_array(ac, 87), -1);
  check_int(bsearch_array(ac, 88), 3);
  check_int(bsearch_array(ac, 89), 4);
  check_int(bsearch_array(ac, 90), -1);
  check_int(bsearch_array(ac, 91), 5);
  check_int(bsearch_array(ac, 92), 6);
  check_int(bsearch_array(ac, 189), -1);
}

/* ---------------------------------------- */

int main(int argc, char **argv) {
  array_container ac;
  char buffer[256];
  int i;
  FILE *f;

  test();

  if (argc != 2) {
    printf("%s: need file for numbers as argument\n", 
           argv[0]);
    return 1;
  }

  ac = make_array_container();
  
  f = fopen(argv[1], "r");
  while (fgets(buffer, 256, f)) {
    add_to_back(ac, atoi(buffer));
  }
  fclose(f);

  while (fgets(buffer, 256, stdin)) {
    i = bsearch_array(ac, atoi(buffer));
    if (i < 0)
      printf("Not there\n");
    else
      printf("At position %d\n", i);
  }

  return 0;
}
